﻿Imports System.IO

Public Class Form1

#Region " Handled Events "

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        PerformBackup()
    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        PerformRestore()
    End Sub

#End Region

#Region " Private Fields "

    Private _ActionPrefix As String = "Backup"

#End Region

#Region " Private Methods "

    ''' <summary>
    ''' Creates a backup file name without overwriting an existing file
    ''' </summary>
    ''' <param name="prefix"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetBackupFileName(ByVal prefix As String) As String
        '-- Establish Locals
        Dim outFolder As String = "c:\temp\"
        Dim r As String = outFolder & prefix & ".bak"
        Dim index As Integer = 1

        '-- See if the backup file already exists
        If File.Exists(r) Then
            While True
                '-- Create the file name
                r = outFolder & prefix & "_" & index.ToString() & ".bak"

                '-- See if the file exists
                If Not File.Exists(r) Then
                    Exit While
                Else
                    index += 1
                End If
            End While
        End If

        '-- Return results
        Return r
    End Function

    ''' <summary>
    ''' Handles the complete event of the backup
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub HandleComplete(ByVal sender As Object, ByVal e As Microsoft.SqlServer.Management.Common.ServerMessageEventArgs)
        lblMsg.Text = _ActionPrefix & " completed..."
        prgProgress.Value = 100

        '-- At this point you can either relocation or package (ZIP) up the backup file to a better storage location
    End Sub

    ''' <summary>
    ''' Handles the percent complete event of the backup
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub HandleProgress(ByVal sender As Object, ByVal e As Microsoft.SqlServer.Management.Smo.PercentCompleteEventArgs)
        lblMsg.Text = "Performing " & _ActionPrefix & ", please wait..."
        lblMsg.Refresh()
        prgProgress.Value = e.Percent
    End Sub

    ''' <summary>
    ''' Perform the backup of the SQL Server
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PerformBackup()
        '-- Establish Locals
        Dim conn As New Microsoft.SqlServer.Management.Common.ServerConnection("localhost")
        Dim myServer As New Microsoft.SqlServer.Management.Smo.Server(conn)
        Dim backup As New Microsoft.SqlServer.Management.Smo.Backup()

        '-- Set the backup actions and settings
        _ActionPrefix = "Backup"
        backup.Action = Microsoft.SqlServer.Management.Smo.BackupActionType.Database
        backup.Database = "StrataFrameSample"
        backup.Initialize = True
        backup.Checksum = True
        backup.ContinueAfterError = True
        backup.Incremental = False

        '-- Clear out the log after a successful backup.  This is a really important step as the log can become huge if it is not truncated.
        backup.LogTruncation = Microsoft.SqlServer.Management.Smo.BackupTruncateLogType.Truncate

        '-- Add a backup device of a physical file on disk.  Just FYI, the device path must be a local path on the SERVER being backed up.  Once
        '   the backup has been completed, you can then move that backup file to some other location if necessary.
        backup.Devices.Add(New Microsoft.SqlServer.Management.Smo.BackupDeviceItem(GetBackupFileName("StrataFrameSample"), Microsoft.SqlServer.Management.Smo.DeviceType.File))

        '-- Create handlers for the backup
        AddHandler backup.PercentComplete, AddressOf HandleProgress
        AddHandler backup.Complete, AddressOf HandleComplete

        '-- Perform the backup on the current thread.  You can also use callbacks and use an asynchonous approach.
        backup.SqlBackup(myServer)
    End Sub

    ''' <summary>
    ''' Perform the restore of the SQL Server
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PerformRestore()
        '-- Set the backup file
        If selectBAKFile.ShowDialog() <> Windows.Forms.DialogResult.OK Then Exit Sub

        '-- Establish Locals
        Dim conn As New Microsoft.SqlServer.Management.Common.ServerConnection("localhost")
        Dim myServer As New Microsoft.SqlServer.Management.Smo.Server(conn)
        Dim restore As New Microsoft.SqlServer.Management.Smo.Restore()

        '-- Set the restore actions and settings
        _ActionPrefix = "Restore"
        restore.Database = "StrataFrameSample"
        restore.ReplaceDatabase = True
        restore.PercentCompleteNotification = 10

        '-- Attempt to get the database if it already exists.  We will do this to kill any processes attached so that
        '   the restore can take place.
        If myServer.Databases("StrataFrameSample") IsNot Nothing Then myServer.KillAllProcesses("StrataFrameSample")

        '-- Set the backup file name and device (which in this case is a file)
        restore.Devices.AddDevice(selectBAKFile.FileName, Microsoft.SqlServer.Management.Smo.DeviceType.File)

        ''-- Since we already know the file groups in this case, we are going to just set them.  You can get more
        ''   sophisticated and restore multiple file groups, enumerate them, etc.  For the purposes of this sample,
        ''   we know what to expect.
        'restore.RelocateFiles.Add(new Microsoft.SqlServer.Management.Smo.RelocateFile(DatabaseFileName, DataFileLocation));


        '-- Create handlers for the restore
        AddHandler restore.PercentComplete, AddressOf HandleProgress
        AddHandler restore.Complete, AddressOf HandleComplete

        '-- Perform the restore on the current thread.  You can also use callbacks and use an asynchonous approach.
        restore.SqlRestore(myServer)
    End Sub

#End Region

    
End Class
